@@ -1,6 +1,6 @@
 //! \file protocol.hpp Defines FastCGI protocol
 /***************************************************************************
-* Copyright (C) 2007 Eddie                                                 *
+* Copyright (C) 2007 Eddie Carle [eddie@erctech.org]                       *
 *                                                                          *
 * This file is part of fastcgi++.                                          *
 *                                                                          *
@@ -22,16 +22,32 @@
 #ifndef PROTOCOL_HPP
 #define PROTOCOL_HPP
 
+#include <fastcgi++/config.h>
 #include <map>
 #include <string>
+#include <exception>
+
+#if defined (HAVE_ENDIAN_H)
 #include <endian.h>
+#elif defined (HAVE_MACHINE_ENDIAN_H)
+#include <machine/endian.h>
+#elif defined (HAVE_ARPA_ENDIAN_H)
+#include <arpa/nameser_compat.h>
+#else
+#error Could not locate a file that defines endianess
+#endif
+
 #include <stdint.h>
 
 #include <boost/shared_array.hpp>
 
+#include <fastcgi++/message.hpp>
+
 //! Topmost namespace for the fastcgi++ library
 namespace Fastcgipp
 {
+	//! Defines the fastcgi++ version
+	extern const char version[];
 	namespace Protocol
 	{
 		//! The request ID of a FastCGI request
@@ -67,7 +83,7 @@ namespace Fastcgipp
 		enum RecordType { BEGIN_REQUEST=1, ABORT_REQUEST=2, END_REQUEST=3, PARAMS=4, IN=5, OUT=6, ERR=7, DATA=8, GET_VALUES=9, GET_VALUES_RESULT=10, UNKNOWN_TYPE=11 };
 		
 		//! Defines text labels for the RecordType values
-		extern char* recordTypeLabels[];
+		extern const char* recordTypeLabels[];
 	}
 }
 		
@@ -337,12 +353,13 @@ namespace Fastcgipp
 		 * up with a pointer/size for both the name and value of the parameter.
 		 *
 		 * @param[in] data Pointer to the record body
+		 * @param[in] dataSize Size of data pointed to by data
 		 * @param[out] name Reference to a pointer that will be pointed to the first byte of the parameter name
 		 * @param[out] nameSize Reference to a value to will be given the size in bytes of the parameter name
 		 * @param[out] value Reference to a pointer that will be pointed to the first byte of the parameter value
 		 * @param[out] valueSize Reference to a value to will be given the size in bytes of the parameter value
 		 */
-		bool processParamHeader(const char* data, size_t dataSize, const char*& name, size_t& nameSize, const char*& value, size_t& valueSize);
+		void processParamHeader(const char* data, size_t dataSize, const char*& name, size_t& nameSize, const char*& value, size_t& valueSize);
 
 		
 		//!Used for the reply of FastCGI management records of type GET_VALUES
@@ -359,8 +376,7 @@ namespace Fastcgipp
 		 * @tparam PADDINGLENGTH Length of padding at the end of the record. This is needed to keep
 		 * the record size a multiple of chunkSize.
 		 */
-		template<int NAMELENGTH, int VALUELENGTH, int PADDINGLENGTH>
-		struct ManagementReply
+		template<int NAMELENGTH, int VALUELENGTH, int PADDINGLENGTH> struct ManagementReply
 		{
 		private:
 			//! Management records header
@@ -406,26 +422,6 @@ namespace Fastcgipp
 		//! Reply record that will be sent when asked if requests can be multiplexed over a single connections
 		extern ManagementReply<15, 1, 8> mpxsConnsReply;
 	}
-
-	//! Data structure used to pass messages within the fastcgi++ task management system
-	/*!
-	 * This data structure is crucial to all operation in the FastCGI library as all
-	 * data passed to requests must be encapsulated in this data structure. A type value
-	 * of 0 means that the message is a FastCGI record and will be processed at a low
-	 * level by the library. Any other type value and the message will be passed up to
-	 * the user to be processed. The data may contain any data that can be casted to/from
-	 * a raw character array. The size obviously represents the exact size of the data
-	 * section.
-	 */
-	struct Message
-	{
-		//! Type of message. A 0 means FastCGI record. Anything else is open.
-		int type;
-		//! Size of the data section.
-		size_t size;
-		//! Pointer to the raw data being passed along with the message.
-		boost::shared_array<char> data;
-	};
 }
 
 #endif
