@@ -1,6 +1,6 @@
 //! \file transceiver.hpp Defines the Fastcgipp::Transceiver class
 /***************************************************************************
-* Copyright (C) 2007 Eddie                                                 *
+* Copyright (C) 2007 Eddie Carle [eddie@erctech.org]                       *
 *                                                                          *
 * This file is part of fastcgi++.                                          *
 *                                                                          *
@@ -64,7 +64,7 @@ namespace Fastcgipp
 		//! Copies pointer and size, not data
 		Block(const Block& block): data(block.data), size(block.size) { }
 		//! Copies pointer and size, not data
-		const Block& operator=(const Block& block) { data=block.data; size=block.size; }
+		const Block& operator=(const Block& block) { data=block.data; size=block.size; return *this; }
 		//! Pointer to start of memory location
 		char* data;
 		//! Size in bytes of memory location
@@ -183,13 +183,15 @@ namespace Fastcgipp
 			std::vector<pollfd>& pollFds;
 			//! A reference to Transceiver::fdBuffer for deleting buffers upon closing of the file descriptor
 			std::map<int, fdBuffer>& fdBuffers;
+			//! Helper function for freeFd(int fd, std::vector<pollfd> pollFds, std::map<int, fdBuffer> fdBuffers)
+			void freeFd(int fd_) { Fastcgipp::Transceiver::freeFd(fd_, pollFds, fdBuffers);  }
 		public:
 			//! Constructor
 			/*!
 			 * @param[out] pollFds_ A reference to Transceiver::pollFds is needed for removing file descriptors when they are closed
 			 * @param[out] fdBuffers_ A reference to Transceiver::fdBuffer is needed for deleting buffers upon closing of the file descriptor
 			 */
-			Buffer(std::vector<pollfd>& pollFds, std::map<int, fdBuffer>& fdBuffers_): pollFds(pollFds), fdBuffers(fdBuffers_), chunks(1), pRead(chunks.begin()->data.get()), writeIt(chunks.begin()) { }
+			Buffer(std::vector<pollfd>& pollFds_, std::map<int, fdBuffer>& fdBuffers_): chunks(1), writeIt(chunks.begin()), pRead(chunks.begin()->data.get()), pollFds(pollFds_), fdBuffers(fdBuffers_)  { }
 
 			//! Request a write block in the buffer
 			/*!
@@ -271,6 +273,23 @@ namespace Fastcgipp
 		
 		//! Transmit all buffered data possible
 		int transmit();
+
+	public:
+		//! Free fd/pipe and all it's associated resources
+		/*!
+		 * By calling this function you close the passed file descriptor
+		 * and free up it's associated buffers and resources. It is safe
+		 * to call this function at any time with any fd(even bad ones).
+		 * If requests still exists with this fd then they will be lost.
+		 *
+		 * @param fd File descriptor to delete/free up
+		 * @param pollFds Epoll container
+		 * @param fdBuffers Container of fd/pipe buffers
+		 */
+		static void freeFd(int fd, std::vector<pollfd>& pollFds, std::map<int, fdBuffer>& fdBuffers);
+
+		//! Helper function for freeFd(int fd, std::vector<pollfd> pollFds, std::map<int, fdBuffer> fdBuffers)
+		void freeFd(int fd_) { freeFd(fd_, pollFds, fdBuffers);  }
 	};
 	
 	//! Predicate for comparing the file descriptor of a pollfd
@@ -286,6 +305,54 @@ namespace Fastcgipp
 	{
 		return x.revents;
 	}
+
+	namespace Exceptions
+	{
+		//! General exception for socket related errors
+		struct Socket: public CodedException
+		{
+			//! Sole Constructor
+			/*!
+			 * @param[in] fd_ File descriptor of socket
+			 * @param[in] erno_ Associated errno
+			 */
+			Socket(const int& fd_, const int& erno_): CodedException(0, erno_), fd(fd_) { }
+			//! File descriptor of socket
+			const int fd;
+		};
+		
+		//! %Exception for write errors to sockets
+		struct SocketWrite: public Socket
+		{
+			//! Sole Constructor
+			/*!
+			 * @param[in] fd_ File descriptor of socket
+			 * @param[in] erno_ Associated errno
+			 */
+			SocketWrite(int fd_, int erno_);
+		};
+		
+		//! %Exception for read errors to sockets
+		struct SocketRead: public Socket
+		{
+			//! Sole Constructor
+			/*!
+			 * @param[in] fd_ File descriptor of socket
+			 * @param[in] erno_ Associated errno
+			 */
+			SocketRead(int fd_, int erno_);
+		};
+		
+		//! %Exception for poll() errors
+		struct SocketPoll: public CodedException
+		{
+			//! Sole Constructor
+			/*!
+			 * @param[in] erno_ Associated errno
+			 */
+			SocketPoll(int erno_);
+		};
+	}
 }
 
 #endif
