@@ -77,7 +77,7 @@ size_t Fastcgipp::Http::percentEscapedToRealBytes(const char* source, char* dest
 			*destination=0;
 			for(int shift=4; shift>=0; shift-=4)
 			{
-				if(++i==size) break;
+				if(++i>=size) break;
 				++source;
 				if((*source|0x20) >= 'a' && (*source|0x20) <= 'f')
 					*destination|=((*source|0x20)-0x57)<<shift;
@@ -95,7 +95,7 @@ size_t Fastcgipp::Http::percentEscapedToRealBytes(const char* source, char* dest
 		else
 			*destination++=*source++;
 
-		if(++i==size) break;
+		if(++i>=size) break;
 	}
 	return destination-start;
 }
@@ -494,7 +494,22 @@ template<class charT> void Fastcgipp::Http::decodeUrlEncoded(const char* data, s
 	size_t valueSize;
 	for(char* i=buffer.get(); i<=buffer.get()+size; ++i)
 	{
-		if(*i == ' ' && nameStart && !valueStart)
+		if(i==buffer.get()+size || *i == fieldSeperator)
+		{
+			if(nameStart && valueStart)
+			{
+				valueSize=percentEscapedToRealBytes(valueStart, valueStart, i-valueStart);
+
+				basic_string<charT> name;
+				charToString(nameStart, nameSize, name);
+				nameStart=i+1;
+				basic_string<charT>& value=output[name];
+				charToString(valueStart, valueSize, value);
+				valueStart=0;
+			}
+		}
+
+		else if(*i == ' ' && nameStart && !valueStart)
 			++nameStart;
 
 		else if(*i == '=' && nameStart && !valueStart)
@@ -502,17 +517,6 @@ template<class charT> void Fastcgipp::Http::decodeUrlEncoded(const char* data, s
 			nameSize=percentEscapedToRealBytes(nameStart, nameStart, i-nameStart);
 			valueStart=i+1;
 		}
-		else if( (i==buffer.get()+size || *i == fieldSeperator) && nameStart && valueStart)
-		{
-			valueSize=percentEscapedToRealBytes(valueStart, valueStart, i-valueStart);
-
-			basic_string<charT> name;
-			charToString(nameStart, nameSize, name);
-			nameStart=i+1;
-			basic_string<charT>& value=output[name];
-			charToString(valueStart, valueSize, value);
-			valueStart=0;
-		}
 	}
 }
 
@@ -640,9 +644,9 @@ void Fastcgipp::Http::Address::assign(const char* start, const char* end)
 		}
 		else if('0' <= *read && *read <= '9')
 			offset = '0';
-		else if('A' <= *read && *read <= 'Z')
+		else if('A' <= *read && *read <= 'F')
 			offset = 'A'-10;
-		else if('a' <= *read && *read <= 'z')
+		else if('a' <= *read && *read <= 'f')
 			offset = 'a'-10;
 		else if(*read == '.')
 		{
@@ -876,9 +880,9 @@ template<class charT, class Traits> std::basic_istream<charT, Traits>& Fastcgipp
 				}
 				else if('0' <= *read && *read <= '9')
 					offset = '0';
-				else if('A' <= *read && *read <= 'Z')
+				else if('A' <= *read && *read <= 'F')
 					offset = 'A'-10;
-				else if('a' <= *read && *read <= 'z')
+				else if('a' <= *read && *read <= 'f')
 					offset = 'a'-10;
 				else if(*read == '.')
 				{
